@@ -18,8 +18,6 @@ use Net::SIP::Debug;
 use Net::SIP::DTMF;
 use Net::SIP::Dispatcher::Eventloop;
 
-my $SSRC = rand()*2**32; # source ID for RTP streams, uniq for each program
-
 # on MSWin32 non-blocking sockets are not supported from IO::Socket
 use constant CAN_NONBLOCKING => $^O ne 'MSWin32';
 
@@ -56,7 +54,7 @@ sub media_recv_echo {
 	    my $echo_back = sub {
 		my ($s_sock,$remote,$delay_buffer,$delay,$writeto,$targs,$didit,$sock) = @_;
 		{
-		    my ($buf,$mpt,$seq,$tstamp,$ssrc,$csrc) =
+		    my (undef,$mpt,undef,$tstamp,undef,undef,$payload) =
 			_receive_rtp($sock,$writeto,$targs,$didit,$channel)
 			or last;
 		    #DEBUG( "$didit=$$didit" );
@@ -64,7 +62,12 @@ sub media_recv_echo {
 
 		    last if ! $s_sock || ! $remote; # call on hold ?
 
-		    my @pkt = _generate_dtmf($targs,$seq,$tstamp,$SSRC);
+		    my $seq = ++($targs->{wseq} //= int(rand(2**16)));
+
+		    # source ID for RTP stream, uniq for each stream
+		    my $ssrc = $targs->{wssrc} //= int(rand(2**32));
+
+		    my @pkt = _generate_dtmf($targs,$seq,$tstamp,$ssrc);
 		    if (@pkt && $pkt[0] ne '') {
 			DEBUG( 100,"send DTMF to RTP");
 			send( $s_sock,$_,0,$remote ) for(@pkt);
@@ -72,6 +75,7 @@ sub media_recv_echo {
 		    }
 
 		    last if $delay<0;
+		    my $buf = pack('CCnNN',0b10000000,$mpt,$seq,$tstamp,$ssrc).$payload;
 		    push @$delay_buffer, $buf;
 		    while ( @$delay_buffer > $delay ) {
 			send( $s_sock,shift(@$delay_buffer),0,$remote );
@@ -240,8 +244,9 @@ sub media_send_recv {
 #   $didit: reference to scalar which gets set to TRUE on each received packet
 #     and which gets set to FALSE from a timer, thus detecting inactivity
 #   $channel: index of RTP channel
-# Return: $packet
+# Return: $packet | $packet,$mpt,$seq,$tstamp,$ssrc,$csrc,$payload
 #   $packet: received RTP packet (including header)
+#   $mpt,$seq,$tstamp,$ssrc,$csrc,$payload - parsed details from RTP header
 ###########################################################################
 sub _receive_rtp {
     my ($sock,$writeto,$targs,$didit,$channel) = @_;
@@ -277,8 +282,8 @@ sub _receive_rtp {
     my $padding = $vpxcc & 0x20 ? unpack( 'C', substr($buf,-1,1)) : 0;
     my $payload = $padding ? substr( $buf,0,length($buf)-$padding ): $buf;
 
-    DEBUG( 100,"ch=%d payload=%d/%d pt=%d xh=%d padding=%d cc=%d",
-	$channel, $seq, length($payload), $mpt & 0x7f, $xh, $padding, $cc);
+    DEBUG( 100,"ch=%d payload=%d/%d pt=%d/%d xh=%d padding=%d cc=%d",
+	$channel, $seq, length($payload), $mpt >> 7, $mpt & 0x7f, $xh, $padding, $cc);
     if ( $targs->{ssrc} && $targs->{ssrc} != $ssrc ) {
 	# RTP stream has changed, reset rseq
 	delete $targs->{rseq};
@@ -311,7 +316,7 @@ sub _receive_rtp {
 	}
     }
 
-    return wantarray ? ( $packet,$mpt,$seq,$tstamp,$ssrc,$csrc ): $packet;
+    return wantarray ? ( $packet,$mpt,$seq,$tstamp,$ssrc,$csrc,$payload ): $packet;
 }
 
 ###########################################################################
@@ -334,13 +339,15 @@ sub _receive_rtp {
 sub _send_rtp {
     my ($sock,$loop,$addr,$readfrom,$channel,$targs,$timer) = @_;
 
-    $targs->{wseq}++;
-    my $seq = $targs->{wseq};
+    my $seq = ++($targs->{wseq} //= int(rand(2**16)));
+
+    # source ID for RTP stream, uniq for each stream
+    my $ssrc = $targs->{wssrc} //= int(rand(2**32));
 
     # 32 bit timestamp based on seq and packet size
     my $timestamp = ( $targs->{rtp_param}[1] * $seq ) % 2**32;
 
-    my @pkt = _generate_dtmf($targs,$seq,$timestamp,$SSRC);
+    my @pkt = _generate_dtmf($targs,$seq,$timestamp,$ssrc);
     if (@pkt && $pkt[0] ne '') {
 	DEBUG( 100,"send DTMF to RTP");
 	send( $sock,$_,0,$addr ) for(@pkt);
@@ -364,7 +371,6 @@ sub _send_rtp {
     } else {
 	# read from file
 	for(my $tries = 0; $tries<2;$tries++ ) {
-	    $targs->{wseq} ||= int( rand( 2**16 ));
 	    my $fd = $targs->{fd};
 	    if ( !$fd ) {
 		$targs->{repeat} = -1 if $targs->{repeat} < 0;
@@ -407,9 +413,10 @@ sub _send_rtp {
 	$payload_type | ( $rtp_event << 7 ) ,
 	$seq, # sequence
 	$timestamp,
-	$SSRC,
+	$ssrc,
     );
-    DEBUG( 100,"send %d bytes to RTP", length($buf));
+    DEBUG( 100,"ch=%d payload=%d/%d pt=%d/%d",
+	$channel, $seq, length($buf), $payload_type >> 7, $payload_type & 0x7f);
     send( $sock,$header.$buf,0,$addr );
 }
 
